/*
** $Header: /u/spert/src/spert-hdrs/RCS/lowmem.h,v 1.8 1996/08/14 02:38:21 davidj Exp $
**
** The definitions of low memory.  This file is based on the old
** spert/os/kernel/kmem.h but with the stuff in t0/headers/kmem.h removed.
** To get the full story, t0/headers/kmem.h must be consulted.
**
** davidj - Wed Feb  8 14:52:07 1995
*/ 

#include <tor/sys/t0kmem.h>

/* Here are the TOHOST and FROMHOST register values used by the kernel */

#define TOHOST_SYSREQ (TOHOST_KERNEL+0)      /* System call request */
#define TOHOST_SIGNAL (TOHOST_KERNEL+1)      /* Signal on T0 */
#define TOHOST_EXITED (TOHOST_KERNEL+2)      /* Program exited */

/* Acknowledgement for TOHOST_SYSREQ */
#define FROMHOST_SYSACK (FROMHOST_KERNEL+0)
/* Acknowledgement for TOHOST_SIGNAL */
#define FROMHOST_SIGACK (FROMHOST_KERNEL+1)


/* This is stuff that is actually put in low memory - start at KMEM_KERNEL,
   which is where the hardware kmem stuff finishes. */

#define KMEM_VARS (KMEM_KERNEL)	/*  */

#define KMEM_MEMTOP (KMEM_VARS+0)
#define KMEM_BRKADDR (KMEM_VARS+4)
#define KMEM_KERNEL_SP (KMEM_VARS+8)   /* Kernel stack pointer */
#define KMEM_PROG_START (KMEM_VARS+12) /* Execution address of program */
#define KMEM_PROG_FTEXT (KMEM_VARS+16) /* Start of text section */
#define KMEM_PROG_ETEXT (KMEM_VARS+20) /* End of text section */
#define KMEM_PROG_FDATA (KMEM_VARS+24) /* Start of data section */
#define KMEM_PROG_EDATA (KMEM_VARS+28) /* End of data section */
#define KMEM_PROG_FBSS (KMEM_VARS+32)  /* Start of bss section */
#define KMEM_PROG_END (KMEM_VARS+36)   /* End of bss section */
#define KMEM_ARGSTART (KMEM_VARS+40)   /* Start of argument block */
#define KMEM_ARGSIZE (KMEM_VARS+44)    /* Size of argument block */
#define KMEM_SIGNAL (KMEM_VARS+48)     /* Signal number from T0 */
#define KMEM_SIGCODE (KMEM_VARS+52)    /* Auxilliary signal code (also used
                                          for exit value) */
#define KMEM_SIGIN (KMEM_VARS+56)      /* Signal from host to T0 */
#define KMEM_STRACE (KMEM_VARS+60)     /* 1 if tracing system calls */
#define KMEM_KERNEL_GP (KMEM_VARS+64)  /* Kernel global pointer */
#define KMEM_SSTEP (KMEM_VARS+68)      /* 1 if single stepping */
#define KMEM_EMU_BRANCH_DEST (KMEM_VARS+72) /* Destination of last emulated
                                               branch instruction */
#define KMEM_POST_DELAY_INST (KMEM_VARS+76) /* Instruction after branch delay
                                               slot of emulated branch
					       inst. */

/* We can send signals to the host by writing to a location with A30 high.  To
   make sure these writes do not write over anything else, we allocate
   a location in low memory and write to a shadow of it. */
#define KMEM_KICKHOST_SPACE (KMEM_VARS+80) 
#define KMEM_KICKHOST_OFFSET (0x40000000)
#define KMEM_KICKHOST_ADDR (KMEM_KICKHOST_SPACE + KMEM_KICKHOST_OFFSET)

/* Free from here on. */
#define KMEM_VARS_SPARE (KMEM_VARS+84)

/* Make sure variables havent overflowed into the message/kernel mem */
#if ((KMEM_VARS_SPARE>=KMEM_KFIXED_DEV)
#error
#endif
