/*
** Some macros for making operating system calls in assembler for SPERT
** davidj - Thu Sep  7 11:01:04 1995
*/

#include <machine/regdef.h>
#include <sys/_spert_syscall.h>


	.extern	_cerror

/****************************************************************
** A macro for a system call routine - performs the system call,
** and sets errno if there is an error.  Used for system calls that return
** a value if everything okay.
**
** Note that a3 is used like a condition flag - non-zero means an error,
** with the error code returned in v0.  Zero means value in v0 is
** normal return value.
*/

#define SYSCALL_RET(NAME,VAL) \
	.globl	NAME; \
	.ent	NAME; \
NAME:; \
	li	v0, VAL; \
	syscall; \
	beqz	a3, 1f; \
	j	_cerror; \
1:; \
	j	ra; \
	.end	NAME

/****************************************************************
** A macro for a system call routine - performs the system call,
** and sets errno if there is an error.  Used for system calls that return
** a zero if everything okay (note that on SPERT we probably dont need
** to clear v0 at end, but this may be useful for testing on Irix).
**
** Note that a3 is used like a condition flag - non-zero means an error,
** with the error code returned in v0.  Zero means value in v0 is
** invalid - must set it to zero.
*/

#define SYSCALL_FLAG(NAME,VAL) \
	.globl	NAME; \
	.ent	NAME; \
NAME:; \
	li	v0, VAL; \
	syscall; \
	beqz	a3, 1f; \
	j	_cerror; \
1:; \
	li	v0, 0; \
	j	ra; \
	.end	NAME
